"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union


class Underwriting(BaseSDK):
    def get(
        self,
        *,
        account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetUnderwritingResponse:
        r"""Retrieve underwriting associated with a given Moov account.

        Read our [underwriting guide](https://docs.moov.io/guides/accounts/requirements/underwriting/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.read` scope.

        :param account_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetUnderwritingRequest(
            account_id=account_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/underwriting",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetUnderwritingGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getUnderwriting",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetUnderwritingResponse(
                result=unmarshal_json_response(components.Underwriting, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetUnderwritingResponse:
        r"""Retrieve underwriting associated with a given Moov account.

        Read our [underwriting guide](https://docs.moov.io/guides/accounts/requirements/underwriting/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.read` scope.

        :param account_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetUnderwritingRequest(
            account_id=account_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/underwriting",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetUnderwritingGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getUnderwriting",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetUnderwritingResponse(
                result=unmarshal_json_response(components.Underwriting, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def save(
        self,
        *,
        account_id: str,
        geographic_reach: Optional[components.GeographicReach] = None,
        business_presence: Optional[components.BusinessPresence] = None,
        pending_litigation: Optional[components.PendingLitigation] = None,
        volume_share_by_customer_type: Optional[
            Union[
                components.VolumeShareByCustomerType,
                components.VolumeShareByCustomerTypeTypedDict,
            ]
        ] = None,
        collect_funds: Optional[
            Union[components.CollectFunds, components.CollectFundsTypedDict]
        ] = None,
        money_transfer: Optional[
            Union[components.MoneyTransfer, components.MoneyTransferTypedDict]
        ] = None,
        send_funds: Optional[
            Union[components.SendFunds, components.SendFundsTypedDict]
        ] = None,
        submission_intent: Optional[components.SubmissionIntent] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.SaveUnderwritingResponse:
        r"""Create or update the account's underwriting.

        Read our [underwriting guide](https://docs.moov.io/guides/accounts/requirements/underwriting/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.write` scope.

        :param account_id:
        :param geographic_reach:
        :param business_presence:
        :param pending_litigation:
        :param volume_share_by_customer_type:
        :param collect_funds:
        :param money_transfer:
        :param send_funds:
        :param submission_intent:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.SaveUnderwritingRequest(
            account_id=account_id,
            upsert_underwriting=components.UpsertUnderwriting(
                geographic_reach=geographic_reach,
                business_presence=business_presence,
                pending_litigation=pending_litigation,
                volume_share_by_customer_type=utils.get_pydantic_model(
                    volume_share_by_customer_type,
                    Optional[components.VolumeShareByCustomerType],
                ),
                collect_funds=utils.get_pydantic_model(
                    collect_funds, Optional[components.CollectFunds]
                ),
                money_transfer=utils.get_pydantic_model(
                    money_transfer, Optional[components.MoneyTransfer]
                ),
                send_funds=utils.get_pydantic_model(
                    send_funds, Optional[components.SendFunds]
                ),
                submission_intent=submission_intent,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounts/{accountID}/underwriting",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.SaveUnderwritingGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.upsert_underwriting,
                False,
                False,
                "json",
                components.UpsertUnderwriting,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="saveUnderwriting",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.SaveUnderwritingResponse(
                result=unmarshal_json_response(components.Underwriting, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UpsertUnderwritingErrorData, http_res
            )
            raise errors.UpsertUnderwritingError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def save_async(
        self,
        *,
        account_id: str,
        geographic_reach: Optional[components.GeographicReach] = None,
        business_presence: Optional[components.BusinessPresence] = None,
        pending_litigation: Optional[components.PendingLitigation] = None,
        volume_share_by_customer_type: Optional[
            Union[
                components.VolumeShareByCustomerType,
                components.VolumeShareByCustomerTypeTypedDict,
            ]
        ] = None,
        collect_funds: Optional[
            Union[components.CollectFunds, components.CollectFundsTypedDict]
        ] = None,
        money_transfer: Optional[
            Union[components.MoneyTransfer, components.MoneyTransferTypedDict]
        ] = None,
        send_funds: Optional[
            Union[components.SendFunds, components.SendFundsTypedDict]
        ] = None,
        submission_intent: Optional[components.SubmissionIntent] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.SaveUnderwritingResponse:
        r"""Create or update the account's underwriting.

        Read our [underwriting guide](https://docs.moov.io/guides/accounts/requirements/underwriting/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.write` scope.

        :param account_id:
        :param geographic_reach:
        :param business_presence:
        :param pending_litigation:
        :param volume_share_by_customer_type:
        :param collect_funds:
        :param money_transfer:
        :param send_funds:
        :param submission_intent:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.SaveUnderwritingRequest(
            account_id=account_id,
            upsert_underwriting=components.UpsertUnderwriting(
                geographic_reach=geographic_reach,
                business_presence=business_presence,
                pending_litigation=pending_litigation,
                volume_share_by_customer_type=utils.get_pydantic_model(
                    volume_share_by_customer_type,
                    Optional[components.VolumeShareByCustomerType],
                ),
                collect_funds=utils.get_pydantic_model(
                    collect_funds, Optional[components.CollectFunds]
                ),
                money_transfer=utils.get_pydantic_model(
                    money_transfer, Optional[components.MoneyTransfer]
                ),
                send_funds=utils.get_pydantic_model(
                    send_funds, Optional[components.SendFunds]
                ),
                submission_intent=submission_intent,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounts/{accountID}/underwriting",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.SaveUnderwritingGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.upsert_underwriting,
                False,
                False,
                "json",
                components.UpsertUnderwriting,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="saveUnderwriting",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.SaveUnderwritingResponse(
                result=unmarshal_json_response(components.Underwriting, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UpsertUnderwritingErrorData, http_res
            )
            raise errors.UpsertUnderwritingError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def upsert(
        self,
        *,
        account_id: str,
        average_transaction_size: int,
        max_transaction_size: int,
        average_monthly_transaction_volume: int,
        volume_by_customer_type: Union[
            components.VolumeByCustomerType, components.VolumeByCustomerTypeTypedDict
        ],
        card_volume_distribution: Union[
            components.CardVolumeDistribution,
            components.CardVolumeDistributionTypedDict,
        ],
        fulfillment: Union[
            components.FulfillmentDetails, components.FulfillmentDetailsTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpsertUnderwritingResponse:
        r"""Create or update the account's underwriting.

        Read our [underwriting guide](https://docs.moov.io/guides/accounts/requirements/underwriting/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.write` scope.

        :param account_id:
        :param average_transaction_size:
        :param max_transaction_size:
        :param average_monthly_transaction_volume:
        :param volume_by_customer_type:
        :param card_volume_distribution:
        :param fulfillment:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpsertUnderwritingRequest(
            account_id=account_id,
            update_underwriting=components.UpdateUnderwriting(
                average_transaction_size=average_transaction_size,
                max_transaction_size=max_transaction_size,
                average_monthly_transaction_volume=average_monthly_transaction_volume,
                volume_by_customer_type=utils.get_pydantic_model(
                    volume_by_customer_type, components.VolumeByCustomerType
                ),
                card_volume_distribution=utils.get_pydantic_model(
                    card_volume_distribution, components.CardVolumeDistribution
                ),
                fulfillment=utils.get_pydantic_model(
                    fulfillment, components.FulfillmentDetails
                ),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/accounts/{accountID}/underwriting",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpsertUnderwritingGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_underwriting,
                False,
                False,
                "json",
                components.UpdateUnderwriting,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="upsertUnderwriting",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpsertUnderwritingResponse(
                result=unmarshal_json_response(components.Underwriting, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UpdateUnderwritingErrorData, http_res
            )
            raise errors.UpdateUnderwritingError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def upsert_async(
        self,
        *,
        account_id: str,
        average_transaction_size: int,
        max_transaction_size: int,
        average_monthly_transaction_volume: int,
        volume_by_customer_type: Union[
            components.VolumeByCustomerType, components.VolumeByCustomerTypeTypedDict
        ],
        card_volume_distribution: Union[
            components.CardVolumeDistribution,
            components.CardVolumeDistributionTypedDict,
        ],
        fulfillment: Union[
            components.FulfillmentDetails, components.FulfillmentDetailsTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpsertUnderwritingResponse:
        r"""Create or update the account's underwriting.

        Read our [underwriting guide](https://docs.moov.io/guides/accounts/requirements/underwriting/) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.write` scope.

        :param account_id:
        :param average_transaction_size:
        :param max_transaction_size:
        :param average_monthly_transaction_volume:
        :param volume_by_customer_type:
        :param card_volume_distribution:
        :param fulfillment:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpsertUnderwritingRequest(
            account_id=account_id,
            update_underwriting=components.UpdateUnderwriting(
                average_transaction_size=average_transaction_size,
                max_transaction_size=max_transaction_size,
                average_monthly_transaction_volume=average_monthly_transaction_volume,
                volume_by_customer_type=utils.get_pydantic_model(
                    volume_by_customer_type, components.VolumeByCustomerType
                ),
                card_volume_distribution=utils.get_pydantic_model(
                    card_volume_distribution, components.CardVolumeDistribution
                ),
                fulfillment=utils.get_pydantic_model(
                    fulfillment, components.FulfillmentDetails
                ),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/accounts/{accountID}/underwriting",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpsertUnderwritingGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_underwriting,
                False,
                False,
                "json",
                components.UpdateUnderwriting,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="upsertUnderwriting",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpsertUnderwritingResponse(
                result=unmarshal_json_response(components.Underwriting, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UpdateUnderwritingErrorData, http_res
            )
            raise errors.UpdateUnderwritingError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class EinTypedDict(TypedDict):
    number: str


class Ein(BaseModel):
    number: str


class TaxIDTypedDict(TypedDict):
    r"""An EIN (employer identification number) for the business. For sole proprietors, an SSN can be used as the EIN."""

    ein: EinTypedDict


class TaxID(BaseModel):
    r"""An EIN (employer identification number) for the business. For sole proprietors, an SSN can be used as the EIN."""

    ein: Ein



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .plaidintegration import PlaidIntegration, PlaidIntegrationTypedDict
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class PlaidPayloadTypedDict(TypedDict):
    r"""Describes the account to link to the Moov account using a Plaid processor token."""

    plaid: PlaidIntegrationTypedDict
    r"""The details of a Plaid processor integration for a linked funding source.

    `sandbox` - When linking a bank account to a `sandbox` account using a Plaid processor token a default bank account
    response will be used. The following default data will be used to generate the bank account in this flow:

    ```
    RoutingNumber: \"011401533\",
    AccountNumber: \"1111222233330000\",
    AccountType:   \"checking\",
    Mask:          \"0000\" 
    ```
    """


class PlaidPayload(BaseModel):
    r"""Describes the account to link to the Moov account using a Plaid processor token."""

    plaid: PlaidIntegration
    r"""The details of a Plaid processor integration for a linked funding source.

    `sandbox` - When linking a bank account to a `sandbox` account using a Plaid processor token a default bank account
    response will be used. The following default data will be used to generate the bank account in this flow:

    ```
    RoutingNumber: \"011401533\",
    AccountNumber: \"1111222233330000\",
    AccountType:   \"checking\",
    Mask:          \"0000\" 
    ```
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CardBrand(str, Enum):
    r"""The card brand."""

    AMERICAN_EXPRESS = "American Express"
    DISCOVER = "Discover"
    MASTERCARD = "Mastercard"
    VISA = "Visa"
    UNKNOWN = "Unknown"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WebhookDataTransferStatus(str, Enum):
    CREATED = "created"
    PENDING = "pending"
    COMPLETED = "completed"
    FAILED = "failed"
    REVERSED = "reversed"
    QUEUED = "queued"
    CANCELED = "canceled"
    SOURCE_COMPLETED = "source.completed"
    SOURCE_CORRECTED = "source.corrected"
    SOURCE_INITIATED = "source.initiated"
    SOURCE_RETURNED = "source.returned"
    SOURCE_ORIGINATED = "source.originated"
    SOURCE_CONFIRMED = "source.confirmed"
    SOURCE_SETTLED = "source.settled"
    SOURCE_FAILED = "source.failed"
    SOURCE_CANCELED = "source.canceled"
    DESTINATION_COMPLETED = "destination.completed"
    DESTINATION_CORRECTED = "destination.corrected"
    DESTINATION_INITIATED = "destination.initiated"
    DESTINATION_RETURNED = "destination.returned"
    DESTINATION_ORIGINATED = "destination.originated"
    DESTINATION_CONFIRMED = "destination.confirmed"
    DESTINATION_FAILED = "destination.failed"
    DESTINATION_ACCEPTED_WITHOUT_POSTING = "destination.accepted-without-posting"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardupdatereason import CardUpdateReason
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WebhookDataCardAutoUpdatedTypedDict(TypedDict):
    card_id: str
    account_id: str
    update_type: CardUpdateReason
    r"""The results of the card update request."""


class WebhookDataCardAutoUpdated(BaseModel):
    card_id: Annotated[str, pydantic.Field(alias="cardID")]

    account_id: Annotated[str, pydantic.Field(alias="accountID")]

    update_type: Annotated[CardUpdateReason, pydantic.Field(alias="updateType")]
    r"""The results of the card update request."""


try:
    WebhookDataCardAutoUpdated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class TerminalApplicationVersionTypedDict(TypedDict):
    r"""Describes a terminal application version."""

    version: str
    r"""The app version of the terminal application (version code for Android terminal application)."""


class TerminalApplicationVersion(BaseModel):
    r"""Describes a terminal application version."""

    version: str
    r"""The app version of the terminal application (version code for Android terminal application)."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class LinkAccountTerminalApplicationTypedDict(TypedDict):
    r"""Describes a request to link an account with a terminal application."""

    terminal_application_id: str
    r"""ID of the terminal application."""


class LinkAccountTerminalApplication(BaseModel):
    r"""Describes a request to link an account with a terminal application."""

    terminal_application_id: Annotated[
        str, pydantic.Field(alias="terminalApplicationID")
    ]
    r"""ID of the terminal application."""


try:
    LinkAccountTerminalApplication.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingcountandamount import BillingCountAndAmount, BillingCountAndAmountTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class InstantPaymentFeesTypedDict(TypedDict):
    r"""A detailed breakdown of instant payment fees."""

    rtp_credit_transaction: BillingCountAndAmountTypedDict
    r"""Fees for RTP credit transactions."""
    push_to_card_transaction: BillingCountAndAmountTypedDict
    r"""Fees for push-to-card transactions."""
    pull_from_card_transaction: BillingCountAndAmountTypedDict
    r"""Fees for pull-from-card transactions."""
    instant_verification: BillingCountAndAmountTypedDict
    r"""Fees for instant payment verifications."""
    total: BillingCountAndAmountTypedDict
    r"""Total instant payment fees."""
    rtp_decline: NotRequired[BillingCountAndAmountTypedDict]
    r"""Fees for RTP declines."""
    push_to_card_decline: NotRequired[BillingCountAndAmountTypedDict]
    r"""Fees for push-to-card declines."""
    pull_from_card_decline: NotRequired[BillingCountAndAmountTypedDict]
    r"""Fees for pull-from-card declines."""
    pull_from_card_refund: NotRequired[BillingCountAndAmountTypedDict]
    r"""Fees for pull-from-card refunds."""


class InstantPaymentFees(BaseModel):
    r"""A detailed breakdown of instant payment fees."""

    rtp_credit_transaction: Annotated[
        BillingCountAndAmount, pydantic.Field(alias="rtpCreditTransaction")
    ]
    r"""Fees for RTP credit transactions."""

    push_to_card_transaction: Annotated[
        BillingCountAndAmount, pydantic.Field(alias="pushToCardTransaction")
    ]
    r"""Fees for push-to-card transactions."""

    pull_from_card_transaction: Annotated[
        BillingCountAndAmount, pydantic.Field(alias="pullFromCardTransaction")
    ]
    r"""Fees for pull-from-card transactions."""

    instant_verification: Annotated[
        BillingCountAndAmount, pydantic.Field(alias="instantVerification")
    ]
    r"""Fees for instant payment verifications."""

    total: BillingCountAndAmount
    r"""Total instant payment fees."""

    rtp_decline: Annotated[
        Optional[BillingCountAndAmount], pydantic.Field(alias="rtpDecline")
    ] = None
    r"""Fees for RTP declines."""

    push_to_card_decline: Annotated[
        Optional[BillingCountAndAmount], pydantic.Field(alias="pushToCardDecline")
    ] = None
    r"""Fees for push-to-card declines."""

    pull_from_card_decline: Annotated[
        Optional[BillingCountAndAmount], pydantic.Field(alias="pullFromCardDecline")
    ] = None
    r"""Fees for pull-from-card declines."""

    pull_from_card_refund: Annotated[
        Optional[BillingCountAndAmount], pydantic.Field(alias="pullFromCardRefund")
    ] = None
    r"""Fees for pull-from-card refunds."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "rtpDecline",
                "pushToCardDecline",
                "pullFromCardDecline",
                "pullFromCardRefund",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    InstantPaymentFees.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class IssuedCardAuthorizationEventResult(str, Enum):
    r"""The result of an event."""

    APPROVED = "approved"
    DECLINED = "declined"
    PROCESSED = "processed"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class RefundPolicy(str, Enum):
    CONDITIONAL_REFUND = "conditional-refund"
    CUSTOM_POLICY = "custom-policy"
    EVENT_BASED_POLICY = "event-based-policy"
    FULL_REFUND_EXTENDED_WINDOW = "full-refund-extended-window"
    FULL_REFUND_WITHIN_30_DAYS = "full-refund-within-30-days"
    NO_REFUNDS = "no-refunds"
    PARTIAL_REFUND = "partial-refund"
    PRORATED_REFUND = "prorated-refund"
    STORE_CREDIT_ONLY = "store-credit-only"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .walletstatus import WalletStatus
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WebhookDataWalletUpdatedTypedDict(TypedDict):
    account_id: str
    wallet_id: str
    status: WalletStatus
    r"""Status of a wallet.
    - `active`: The wallet is available for use and has an enabled payment method.
    - `closed`: The wallet is no longer active and the corresponding payment method has been disabled.
    """


class WebhookDataWalletUpdated(BaseModel):
    account_id: Annotated[str, pydantic.Field(alias="accountID")]

    wallet_id: Annotated[str, pydantic.Field(alias="walletID")]

    status: WalletStatus
    r"""Status of a wallet.
    - `active`: The wallet is available for use and has an enabled payment method.
    - `closed`: The wallet is no longer active and the corresponding payment method has been disabled.
    """


try:
    WebhookDataWalletUpdated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from moovio_sdk.models.components import (
    wallettransaction as components_wallettransaction,
    wallettransactionsourcetype as components_wallettransactionsourcetype,
    wallettransactionstatus as components_wallettransactionstatus,
    wallettransactiontype as components_wallettransactiontype,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from moovio_sdk.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListWalletTransactionsGlobalsTypedDict(TypedDict):
    x_moov_version: NotRequired[str]
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """


class ListWalletTransactionsGlobals(BaseModel):
    x_moov_version: Annotated[
        Optional[str],
        pydantic.Field(alias="X-Moov-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["X-Moov-Version"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListWalletTransactionsRequestTypedDict(TypedDict):
    account_id: str
    wallet_id: str
    skip: NotRequired[int]
    count: NotRequired[int]
    transaction_type: NotRequired[
        components_wallettransactiontype.WalletTransactionType
    ]
    r"""Optional parameter to filter by transaction type."""
    transaction_types: NotRequired[
        List[components_wallettransactiontype.WalletTransactionType]
    ]
    r"""Optional, comma-separated parameter to filter by transaction types."""
    source_type: NotRequired[
        components_wallettransactionsourcetype.WalletTransactionSourceType
    ]
    r"""Optional parameter to filter by source type (i.e. transfer, dispute, issuing-transaction)."""
    source_id: NotRequired[str]
    r"""Optional parameter to filter by source ID."""
    status: NotRequired[components_wallettransactionstatus.WalletTransactionStatus]
    r"""Optional parameter to filter by status (`pending` or `completed`)."""
    created_start_date_time: NotRequired[datetime]
    r"""Optional date-time which inclusively filters all transactions created after this date-time."""
    created_end_date_time: NotRequired[datetime]
    r"""Optional date-time which exclusively filters all transactions created before this date-time."""
    completed_start_date_time: NotRequired[datetime]
    r"""Optional date-time which inclusively filters all transactions completed after this date-time."""
    completed_end_date_time: NotRequired[datetime]
    r"""Optional date-time which exclusively filters all transactions completed before this date-time."""
    sweep_id: NotRequired[str]
    r"""Optional ID to filter for transactions accrued in a sweep."""


class ListWalletTransactionsRequest(BaseModel):
    account_id: Annotated[
        str,
        pydantic.Field(alias="accountID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    wallet_id: Annotated[
        str,
        pydantic.Field(alias="walletID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    skip: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    count: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    transaction_type: Annotated[
        Optional[components_wallettransactiontype.WalletTransactionType],
        pydantic.Field(alias="transactionType"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional parameter to filter by transaction type."""

    transaction_types: Annotated[
        Optional[List[components_wallettransactiontype.WalletTransactionType]],
        pydantic.Field(alias="transactionTypes"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional, comma-separated parameter to filter by transaction types."""

    source_type: Annotated[
        Optional[components_wallettransactionsourcetype.WalletTransactionSourceType],
        pydantic.Field(alias="sourceType"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional parameter to filter by source type (i.e. transfer, dispute, issuing-transaction)."""

    source_id: Annotated[
        Optional[str],
        pydantic.Field(alias="sourceID"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional parameter to filter by source ID."""

    status: Annotated[
        Optional[components_wallettransactionstatus.WalletTransactionStatus],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional parameter to filter by status (`pending` or `completed`)."""

    created_start_date_time: Annotated[
        Optional[datetime],
        pydantic.Field(alias="createdStartDateTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional date-time which inclusively filters all transactions created after this date-time."""

    created_end_date_time: Annotated[
        Optional[datetime],
        pydantic.Field(alias="createdEndDateTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional date-time which exclusively filters all transactions created before this date-time."""

    completed_start_date_time: Annotated[
        Optional[datetime],
        pydantic.Field(alias="completedStartDateTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional date-time which inclusively filters all transactions completed after this date-time."""

    completed_end_date_time: Annotated[
        Optional[datetime],
        pydantic.Field(alias="completedEndDateTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional date-time which exclusively filters all transactions completed before this date-time."""

    sweep_id: Annotated[
        Optional[str],
        pydantic.Field(alias="sweepID"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Optional ID to filter for transactions accrued in a sweep."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "skip",
                "count",
                "transactionType",
                "transactionTypes",
                "sourceType",
                "sourceID",
                "status",
                "createdStartDateTime",
                "createdEndDateTime",
                "completedStartDateTime",
                "completedEndDateTime",
                "sweepID",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListWalletTransactionsResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: List[components_wallettransaction.WalletTransactionTypedDict]


class ListWalletTransactionsResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: List[components_wallettransaction.WalletTransaction]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DisbursementPaymentMethodType(str, Enum):
    r"""Payment methods allowed for disbursing funds."""

    PUSH_TO_CARD = "push-to-card"
    RTP_CREDIT = "rtp-credit"
    ACH_CREDIT_SAME_DAY = "ach-credit-same-day"
    ACH_CREDIT_STANDARD = "ach-credit-standard"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
import importlib
from moovio_sdk import utils
from moovio_sdk._hooks import SDKHooks
from moovio_sdk.models import components, internal
from moovio_sdk.types import OptionalNullable, UNSET
import sys
from typing import Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from moovio_sdk.account_terminal_applications import AccountTerminalApplications
    from moovio_sdk.accounts import Accounts
    from moovio_sdk.adjustments import Adjustments
    from moovio_sdk.apple_pay import ApplePay
    from moovio_sdk.authentication import Authentication
    from moovio_sdk.avatars import Avatars
    from moovio_sdk.bank_accounts import BankAccounts
    from moovio_sdk.branding import Branding
    from moovio_sdk.capabilities import Capabilities
    from moovio_sdk.card_issuing import CardIssuing
    from moovio_sdk.cards import Cards
    from moovio_sdk.disputes import Disputes
    from moovio_sdk.end_to_end_encryption import EndToEndEncryption
    from moovio_sdk.enriched_address import EnrichedAddress
    from moovio_sdk.enriched_profile import EnrichedProfile
    from moovio_sdk.fee_plans import FeePlans
    from moovio_sdk.files import Files
    from moovio_sdk.images import Images
    from moovio_sdk.industries import Industries
    from moovio_sdk.institutions import Institutions
    from moovio_sdk.invoices import Invoices
    from moovio_sdk.issuing_transactions import IssuingTransactions
    from moovio_sdk.onboarding import Onboarding
    from moovio_sdk.payment_links import PaymentLinks
    from moovio_sdk.payment_methods import PaymentMethods
    from moovio_sdk.ping import Ping
    from moovio_sdk.products import Products
    from moovio_sdk.receipts import Receipts
    from moovio_sdk.representatives import Representatives
    from moovio_sdk.scheduling import Scheduling
    from moovio_sdk.statements import Statements
    from moovio_sdk.support import Support
    from moovio_sdk.sweeps import Sweeps
    from moovio_sdk.terminal_applications import TerminalApplications
    from moovio_sdk.transfers import Transfers
    from moovio_sdk.underwriting import Underwriting
    from moovio_sdk.wallet_transactions import WalletTransactions
    from moovio_sdk.wallets import Wallets
    from moovio_sdk.webhooks import Webhooks


class Moov(BaseSDK):
    r"""Moov API: Moov is a platform that enables developers to integrate all aspects of money movement with ease and speed.
    The Moov API makes it simple for platforms to send, receive, and store money. Our API is based upon REST
    principles, returns JSON responses, and uses standard HTTP response codes. To learn more about how Moov
    works at a high level, read our [concepts](https://docs.moov.io/guides/get-started/glossary/) guide.
    https://docs.moov.io/ - Moov Guides and API Documentation
    """

    accounts: "Accounts"
    adjustments: "Adjustments"
    apple_pay: "ApplePay"
    bank_accounts: "BankAccounts"
    branding: "Branding"
    capabilities: "Capabilities"
    cards: "Cards"
    disputes: "Disputes"
    fee_plans: "FeePlans"
    files: "Files"
    images: "Images"
    invoices: "Invoices"
    payment_links: "PaymentLinks"
    payment_methods: "PaymentMethods"
    products: "Products"
    representatives: "Representatives"
    scheduling: "Scheduling"
    statements: "Statements"
    sweeps: "Sweeps"
    account_terminal_applications: "AccountTerminalApplications"
    support: "Support"
    transfers: "Transfers"
    underwriting: "Underwriting"
    wallets: "Wallets"
    wallet_transactions: "WalletTransactions"
    avatars: "Avatars"
    end_to_end_encryption: "EndToEndEncryption"
    enriched_address: "EnrichedAddress"
    enriched_profile: "EnrichedProfile"
    webhooks: "Webhooks"
    industries: "Industries"
    institutions: "Institutions"
    issuing_transactions: "IssuingTransactions"
    card_issuing: "CardIssuing"
    authentication: "Authentication"
    onboarding: "Onboarding"
    ping: "Ping"
    receipts: "Receipts"
    terminal_applications: "TerminalApplications"
    _sub_sdk_map = {
        "accounts": ("moovio_sdk.accounts", "Accounts"),
        "adjustments": ("moovio_sdk.adjustments", "Adjustments"),
        "apple_pay": ("moovio_sdk.apple_pay", "ApplePay"),
        "bank_accounts": ("moovio_sdk.bank_accounts", "BankAccounts"),
        "branding": ("moovio_sdk.branding", "Branding"),
        "capabilities": ("moovio_sdk.capabilities", "Capabilities"),
        "cards": ("moovio_sdk.cards", "Cards"),
        "disputes": ("moovio_sdk.disputes", "Disputes"),
        "fee_plans": ("moovio_sdk.fee_plans", "FeePlans"),
        "files": ("moovio_sdk.files", "Files"),
        "images": ("moovio_sdk.images", "Images"),
        "invoices": ("moovio_sdk.invoices", "Invoices"),
        "payment_links": ("moovio_sdk.payment_links", "PaymentLinks"),
        "payment_methods": ("moovio_sdk.payment_methods", "PaymentMethods"),
        "products": ("moovio_sdk.products", "Products"),
        "representatives": ("moovio_sdk.representatives", "Representatives"),
        "scheduling": ("moovio_sdk.scheduling", "Scheduling"),
        "statements": ("moovio_sdk.statements", "Statements"),
        "sweeps": ("moovio_sdk.sweeps", "Sweeps"),
        "account_terminal_applications": (
            "moovio_sdk.account_terminal_applications",
            "AccountTerminalApplications",
        ),
        "support": ("moovio_sdk.support", "Support"),
        "transfers": ("moovio_sdk.transfers", "Transfers"),
        "underwriting": ("moovio_sdk.underwriting", "Underwriting"),
        "wallets": ("moovio_sdk.wallets", "Wallets"),
        "wallet_transactions": ("moovio_sdk.wallet_transactions", "WalletTransactions"),
        "avatars": ("moovio_sdk.avatars", "Avatars"),
        "end_to_end_encryption": (
            "moovio_sdk.end_to_end_encryption",
            "EndToEndEncryption",
        ),
        "enriched_address": ("moovio_sdk.enriched_address", "EnrichedAddress"),
        "enriched_profile": ("moovio_sdk.enriched_profile", "EnrichedProfile"),
        "webhooks": ("moovio_sdk.webhooks", "Webhooks"),
        "industries": ("moovio_sdk.industries", "Industries"),
        "institutions": ("moovio_sdk.institutions", "Institutions"),
        "issuing_transactions": (
            "moovio_sdk.issuing_transactions",
            "IssuingTransactions",
        ),
        "card_issuing": ("moovio_sdk.card_issuing", "CardIssuing"),
        "authentication": ("moovio_sdk.authentication", "Authentication"),
        "onboarding": ("moovio_sdk.onboarding", "Onboarding"),
        "ping": ("moovio_sdk.ping", "Ping"),
        "receipts": ("moovio_sdk.receipts", "Receipts"),
        "terminal_applications": (
            "moovio_sdk.terminal_applications",
            "TerminalApplications",
        ),
    }

    def __init__(
        self,
        security: Optional[
            Union[components.Security, Callable[[], components.Security]]
        ] = None,
        x_moov_version: Optional[str] = None,
        server_idx: Optional[int] = None,
        url_params: Optional[Dict[str, str]] = None,
        server_url: Optional[str] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param security: The security details required for authentication
        :param x_moov_version: Configures the x_moov_version parameter for all supported operations
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client(follow_redirects=True)
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient(follow_redirects=True)
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        _globals = internal.Globals(
            x_moov_version=utils.get_global_from_env(
                x_moov_version, "MOOV_X_MOOV_VERSION", str
            ),
        )

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                globals=_globals,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self.sdk_configuration = hooks.sdk_init(self.sdk_configuration)

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .granttype import GrantType
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from moovio_sdk.utils import FieldMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AuthTokenRequestTypedDict(TypedDict):
    grant_type: GrantType
    r"""The type of grant being requested.

    - `client_credentials`: A grant type used by clients to obtain an access token
    - `refresh_token`: A grant type used by clients to obtain a new access token using a refresh token
    """
    client_id: NotRequired[str]
    r"""Client ID can be provided here in the body, or as the Username in HTTP Basic Auth."""
    client_secret: NotRequired[str]
    r"""Client secret can be provided here in the body, or as the Password in HTTP Basic Auth."""
    scope: NotRequired[str]
    r"""A space delimited list of scopes. Required when `grant_type` is `client_credentials`."""
    refresh_token: NotRequired[str]
    r"""The refresh_token returned alongside the access token being refreshed. Required when `grant_type` is `refresh_token`."""


class AuthTokenRequest(BaseModel):
    grant_type: Annotated[GrantType, FieldMetadata(form=True)]
    r"""The type of grant being requested.

    - `client_credentials`: A grant type used by clients to obtain an access token
    - `refresh_token`: A grant type used by clients to obtain a new access token using a refresh token
    """

    client_id: Annotated[Optional[str], FieldMetadata(form=True)] = None
    r"""Client ID can be provided here in the body, or as the Username in HTTP Basic Auth."""

    client_secret: Annotated[Optional[str], FieldMetadata(form=True)] = None
    r"""Client secret can be provided here in the body, or as the Password in HTTP Basic Auth."""

    scope: Annotated[Optional[str], FieldMetadata(form=True)] = None
    r"""A space delimited list of scopes. Required when `grant_type` is `client_credentials`."""

    refresh_token: Annotated[Optional[str], FieldMetadata(form=True)] = None
    r"""The refresh_token returned alongside the access token being refreshed. Required when `grant_type` is `refresh_token`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["client_id", "client_secret", "scope", "refresh_token"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .issuingintervallimit import IssuingIntervalLimit
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class IssuingVelocityLimitTypedDict(TypedDict):
    amount: int
    r"""The maximum amount in cents that can be spent in a given interval."""
    interval: IssuingIntervalLimit
    r"""Specifies the time frame for the velocity limit. Currently supports only per-transaction limits."""


class IssuingVelocityLimit(BaseModel):
    amount: int
    r"""The maximum amount in cents that can be spent in a given interval."""

    interval: IssuingIntervalLimit
    r"""Specifies the time frame for the velocity limit. Currently supports only per-transaction limits."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class AchLocationTypedDict(TypedDict):
    address: str
    city: str
    postal_code: str
    postal_code_extension: str
    state: str


class AchLocation(BaseModel):
    address: str

    city: str

    postal_code: Annotated[str, pydantic.Field(alias="postalCode")]

    postal_code_extension: Annotated[str, pydantic.Field(alias="postalCodeExtension")]

    state: str


try:
    AchLocation.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class FilePurpose(str, Enum):
    r"""The file's purpose."""

    BUSINESS_VERIFICATION = "business_verification"
    REPRESENTATIVE_VERIFICATION = "representative_verification"
    INDIVIDUAL_VERIFICATION = "individual_verification"
    MERCHANT_UNDERWRITING = "merchant_underwriting"
    ACCOUNT_REQUIREMENT = "account_requirement"
    IDENTITY_VERIFICATION = "identity_verification"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createauthorizeduserupdate import (
    CreateAuthorizedUserUpdate,
    CreateAuthorizedUserUpdateTypedDict,
)
from .updateissuedcardstate import UpdateIssuedCardState
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateIssuedCardTypedDict(TypedDict):
    state: NotRequired[UpdateIssuedCardState]
    r"""Updates the state of a Moov issued card.
    - `closed`: The card is permanently deactivated and cannot approve authorizations. A card can be closed by request or when it expires.
    """
    memo: NotRequired[str]
    authorized_user: NotRequired[CreateAuthorizedUserUpdateTypedDict]
    r"""Fields for identifying an authorized individual."""


class UpdateIssuedCard(BaseModel):
    state: Optional[UpdateIssuedCardState] = None
    r"""Updates the state of a Moov issued card.
    - `closed`: The card is permanently deactivated and cannot approve authorizations. A card can be closed by request or when it expires.
    """

    memo: Optional[str] = None

    authorized_user: Annotated[
        Optional[CreateAuthorizedUserUpdate], pydantic.Field(alias="authorizedUser")
    ] = None
    r"""Fields for identifying an authorized individual."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["state", "memo", "authorizedUser"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    UpdateIssuedCard.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .mode import Mode
from .occurrencestatus import OccurrenceStatus
from .runtransfer import RunTransfer, RunTransferTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ErrorTypedDict(TypedDict):
    r"""Contains details on why the occurrence errored."""

    message: NotRequired[str]


class Error(BaseModel):
    r"""Contains details on why the occurrence errored."""

    message: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["message"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class OccurrencesResponseTypedDict(TypedDict):
    run_on: datetime
    run_transfer: RunTransferTypedDict
    schedule_id: NotRequired[str]
    occurrence_id: NotRequired[str]
    mode: NotRequired[Mode]
    r"""Sandbox or production account mode of this schedule."""
    generated: NotRequired[bool]
    r"""True if this was generated by a RRule."""
    indefinite: NotRequired[bool]
    r"""True if the RRule set runs indefinitely."""
    canceled_on: NotRequired[datetime]
    ran_on: NotRequired[datetime]
    ran_transfer_id: NotRequired[str]
    status: NotRequired[OccurrenceStatus]
    r"""Status of the completed occurrence."""
    error: NotRequired[ErrorTypedDict]
    r"""Contains details on why the occurrence errored."""


class OccurrencesResponse(BaseModel):
    run_on: Annotated[datetime, pydantic.Field(alias="runOn")]

    run_transfer: Annotated[RunTransfer, pydantic.Field(alias="runTransfer")]

    schedule_id: Annotated[Optional[str], pydantic.Field(alias="scheduleID")] = None

    occurrence_id: Annotated[Optional[str], pydantic.Field(alias="occurrenceID")] = None

    mode: Optional[Mode] = None
    r"""Sandbox or production account mode of this schedule."""

    generated: Optional[bool] = None
    r"""True if this was generated by a RRule."""

    indefinite: Optional[bool] = None
    r"""True if the RRule set runs indefinitely."""

    canceled_on: Annotated[Optional[datetime], pydantic.Field(alias="canceledOn")] = (
        None
    )

    ran_on: Annotated[Optional[datetime], pydantic.Field(alias="ranOn")] = None

    ran_transfer_id: Annotated[Optional[str], pydantic.Field(alias="ranTransferID")] = (
        None
    )

    status: Optional[OccurrenceStatus] = None
    r"""Status of the completed occurrence."""

    error: Optional[Error] = None
    r"""Contains details on why the occurrence errored."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "scheduleID",
                "occurrenceID",
                "mode",
                "generated",
                "indefinite",
                "canceledOn",
                "ranOn",
                "ranTransferID",
                "status",
                "error",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    OccurrencesResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from typing_extensions import deprecated


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class AccountVerificationStatus(str, Enum):
    r"""Possible states an account verification can be in."""

    UNVERIFIED = "unverified"
    PENDING = "pending"
    RESUBMIT = "resubmit"
    REVIEW = "review"
    VERIFIED = "verified"
    FAILED = "failed"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AppleBillingContactTypedDict(TypedDict):
    r"""Billing contact information as returned from Apple Pay.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymentcontact)
    for more information.
    """

    address_lines: NotRequired[List[str]]
    r"""Address lines 1 and 2 for the contact."""
    locality: NotRequired[str]
    r"""The contact's city."""
    postal_code: NotRequired[str]
    r"""The contact's postal code."""
    administrative_area: NotRequired[str]
    r"""The contact's two-letter state code."""
    country_code: NotRequired[str]
    r"""The contact's two-letter ISO 3166"""


class AppleBillingContact(BaseModel):
    r"""Billing contact information as returned from Apple Pay.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymentcontact)
    for more information.
    """

    address_lines: Annotated[
        Optional[List[str]], pydantic.Field(alias="addressLines")
    ] = None
    r"""Address lines 1 and 2 for the contact."""

    locality: Optional[str] = None
    r"""The contact's city."""

    postal_code: Annotated[Optional[str], pydantic.Field(alias="postalCode")] = None
    r"""The contact's postal code."""

    administrative_area: Annotated[
        Optional[str], pydantic.Field(alias="administrativeArea")
    ] = None
    r"""The contact's two-letter state code."""

    country_code: Annotated[Optional[str], pydantic.Field(alias="countryCode")] = None
    r"""The contact's two-letter ISO 3166"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "addressLines",
                "locality",
                "postalCode",
                "administrativeArea",
                "countryCode",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    AppleBillingContact.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cancellationstatus import CancellationStatus
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WebhookDataCancellationCreatedTypedDict(TypedDict):
    cancellation_id: str
    transfer_id: str
    status: CancellationStatus


class WebhookDataCancellationCreated(BaseModel):
    cancellation_id: Annotated[str, pydantic.Field(alias="cancellationID")]

    transfer_id: Annotated[str, pydantic.Field(alias="transferID")]

    status: CancellationStatus


try:
    WebhookDataCancellationCreated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class IssuedCardState(str, Enum):
    r"""The `state` represents the operational status of an issued card. A card can only approve incoming authorizations if it is in an active state.

    - `active`: The card is operational and approves authorizations. Generally becomes active shortly after card creation.
    - `inactive`: The card cannot approve authorizations. This is currently a temporary state assigned post-creation during the activation process.
    - `closed`: The card is permanently deactivated and cannot approve authorizations. A card can be closed by request or when it expires.
    - `pending-verification`: Awaiting additional authorized user verification before the card can be activated.
    """

    ACTIVE = "active"
    INACTIVE = "inactive"
    PENDING_VERIFICATION = "pending-verification"
    CLOSED = "closed"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CompleteMicroDepositsTypedDict(TypedDict):
    r"""Request to complete the micro-deposit verification workflow."""

    amounts: List[int]
    r"""Two positive integers, in cents, equal to the values of the micro-deposits sent to the bank account."""


class CompleteMicroDeposits(BaseModel):
    r"""Request to complete the micro-deposit verification workflow."""

    amounts: List[int]
    r"""Two positive integers, in cents, equal to the values of the micro-deposits sent to the bank account."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.components import (
    cardvolumedistributionerror as components_cardvolumedistributionerror,
    collectfundserror as components_collectfundserror,
    fulfillmentdetailserror as components_fulfillmentdetailserror,
    moneytransfererror as components_moneytransfererror,
    sendfundserror as components_sendfundserror,
    volumebycustomertypeerror as components_volumebycustomertypeerror,
    volumesharebycustomertypeerror as components_volumesharebycustomertypeerror,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class UpsertUnderwritingErrorData(BaseModel):
    geographic_reach: Annotated[
        Optional[str], pydantic.Field(alias="geographicReach")
    ] = None
    business_presence: Annotated[
        Optional[str], pydantic.Field(alias="businessPresence")
    ] = None
    pending_litigation: Annotated[
        Optional[str], pydantic.Field(alias="pendingLitigation")
    ] = None
    volume_share_by_customer_type: Annotated[
        Optional[
            components_volumesharebycustomertypeerror.VolumeShareByCustomerTypeError
        ],
        pydantic.Field(alias="volumeShareByCustomerType"),
    ] = None
    collect_funds: Annotated[
        Optional[components_collectfundserror.CollectFundsError],
        pydantic.Field(alias="collectFunds"),
    ] = None
    money_transfer: Annotated[
        Optional[components_moneytransfererror.MoneyTransferError],
        pydantic.Field(alias="moneyTransfer"),
    ] = None
    send_funds: Annotated[
        Optional[components_sendfundserror.SendFundsError],
        pydantic.Field(alias="sendFunds"),
    ] = None
    average_monthly_transaction_volume: Annotated[
        Optional[str], pydantic.Field(alias="averageMonthlyTransactionVolume")
    ] = None
    error: Optional[str] = None
    average_transaction_size: Annotated[
        Optional[str], pydantic.Field(alias="averageTransactionSize")
    ] = None
    max_transaction_size: Annotated[
        Optional[str], pydantic.Field(alias="maxTransactionSize")
    ] = None
    volume_by_customer_type: Annotated[
        Optional[components_volumebycustomertypeerror.VolumeByCustomerTypeError],
        pydantic.Field(alias="volumeByCustomerType"),
    ] = None
    card_volume_distribution: Annotated[
        Optional[components_cardvolumedistributionerror.CardVolumeDistributionError],
        pydantic.Field(alias="cardVolumeDistribution"),
    ] = None
    fulfillment: Optional[
        components_fulfillmentdetailserror.FulfillmentDetailsError
    ] = None


@dataclass(unsafe_hash=True)
class UpsertUnderwritingError(MoovError):
    data: UpsertUnderwritingErrorData = field(hash=False)

    def __init__(
        self,
        data: UpsertUnderwritingErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .businesspresence import BusinessPresence
from .cardvolumedistribution import (
    CardVolumeDistribution,
    CardVolumeDistributionTypedDict,
)
from .collectfunds import CollectFunds, CollectFundsTypedDict
from .fulfillmentdetails import FulfillmentDetails, FulfillmentDetailsTypedDict
from .geographicreach import GeographicReach
from .moneytransfer import MoneyTransfer, MoneyTransferTypedDict
from .pendinglitigation import PendingLitigation
from .sendfunds import SendFunds, SendFundsTypedDict
from .underwritingstatus import UnderwritingStatus
from .volumebycustomertype import VolumeByCustomerType, VolumeByCustomerTypeTypedDict
from .volumesharebycustomertype import (
    VolumeShareByCustomerType,
    VolumeShareByCustomerTypeTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UnderwritingTypedDict(TypedDict):
    r"""Describes underwriting values (in USD) used for card payment acceptance."""

    average_transaction_size: NotRequired[int]
    max_transaction_size: NotRequired[int]
    average_monthly_transaction_volume: NotRequired[int]
    status: NotRequired[UnderwritingStatus]
    volume_by_customer_type: NotRequired[VolumeByCustomerTypeTypedDict]
    card_volume_distribution: NotRequired[CardVolumeDistributionTypedDict]
    fulfillment: NotRequired[FulfillmentDetailsTypedDict]
    geographic_reach: NotRequired[GeographicReach]
    business_presence: NotRequired[BusinessPresence]
    pending_litigation: NotRequired[PendingLitigation]
    volume_share_by_customer_type: NotRequired[VolumeShareByCustomerTypeTypedDict]
    collect_funds: NotRequired[CollectFundsTypedDict]
    money_transfer: NotRequired[MoneyTransferTypedDict]
    send_funds: NotRequired[SendFundsTypedDict]


class Underwriting(BaseModel):
    r"""Describes underwriting values (in USD) used for card payment acceptance."""

    average_transaction_size: Annotated[
        Optional[int], pydantic.Field(alias="averageTransactionSize")
    ] = None

    max_transaction_size: Annotated[
        Optional[int], pydantic.Field(alias="maxTransactionSize")
    ] = None

    average_monthly_transaction_volume: Annotated[
        Optional[int], pydantic.Field(alias="averageMonthlyTransactionVolume")
    ] = None

    status: Annotated[
        Optional[UnderwritingStatus],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None

    volume_by_customer_type: Annotated[
        Optional[VolumeByCustomerType], pydantic.Field(alias="volumeByCustomerType")
    ] = None

    card_volume_distribution: Annotated[
        Optional[CardVolumeDistribution], pydantic.Field(alias="cardVolumeDistribution")
    ] = None

    fulfillment: Optional[FulfillmentDetails] = None

    geographic_reach: Annotated[
        Optional[GeographicReach], pydantic.Field(alias="geographicReach")
    ] = None

    business_presence: Annotated[
        Optional[BusinessPresence], pydantic.Field(alias="businessPresence")
    ] = None

    pending_litigation: Annotated[
        Optional[PendingLitigation], pydantic.Field(alias="pendingLitigation")
    ] = None

    volume_share_by_customer_type: Annotated[
        Optional[VolumeShareByCustomerType],
        pydantic.Field(alias="volumeShareByCustomerType"),
    ] = None

    collect_funds: Annotated[
        Optional[CollectFunds], pydantic.Field(alias="collectFunds")
    ] = None

    money_transfer: Annotated[
        Optional[MoneyTransfer], pydantic.Field(alias="moneyTransfer")
    ] = None

    send_funds: Annotated[Optional[SendFunds], pydantic.Field(alias="sendFunds")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "averageTransactionSize",
                "maxTransactionSize",
                "averageMonthlyTransactionVolume",
                "status",
                "volumeByCustomerType",
                "cardVolumeDistribution",
                "fulfillment",
                "geographicReach",
                "businessPresence",
                "pendingLitigation",
                "volumeShareByCustomerType",
                "collectFunds",
                "moneyTransfer",
                "sendFunds",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Underwriting.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cancellationstatus import CancellationStatus
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class CancellationTypedDict(TypedDict):
    cancellation_id: str
    status: CancellationStatus
    created_on: datetime


class Cancellation(BaseModel):
    cancellation_id: Annotated[str, pydantic.Field(alias="cancellationID")]

    status: CancellationStatus

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]


try:
    Cancellation.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billablefee import BillableFee, BillableFeeTypedDict
from .cardacquiringmodel import CardAcquiringModel
from .minimumcommitment import MinimumCommitment, MinimumCommitmentTypedDict
from .monthlyplatformfee import MonthlyPlatformFee, MonthlyPlatformFeeTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FeePlanTypedDict(TypedDict):
    plan_id: str
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    name: str
    r"""The name of the fee plan."""
    card_acquiring_model: CardAcquiringModel
    r"""Specifies the card processing pricing model"""
    billable_fees: List[BillableFeeTypedDict]
    r"""Additional usage-based fees for this plan."""
    minimum_commitment: MinimumCommitmentTypedDict
    r"""The minimum spending amount that must be met in the billing period. If actual usage is below the minimum amount, account is charged the difference."""
    monthly_platform_fee: MonthlyPlatformFeeTypedDict
    r"""Fixed recurring amount paid in the billing period regardless of usage."""
    created_at: datetime
    description: NotRequired[str]
    r"""A description of the fee plan."""


class FeePlan(BaseModel):
    plan_id: Annotated[str, pydantic.Field(alias="planID")]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    name: str
    r"""The name of the fee plan."""

    card_acquiring_model: Annotated[
        CardAcquiringModel, pydantic.Field(alias="cardAcquiringModel")
    ]
    r"""Specifies the card processing pricing model"""

    billable_fees: Annotated[List[BillableFee], pydantic.Field(alias="billableFees")]
    r"""Additional usage-based fees for this plan."""

    minimum_commitment: Annotated[
        MinimumCommitment, pydantic.Field(alias="minimumCommitment")
    ]
    r"""The minimum spending amount that must be met in the billing period. If actual usage is below the minimum amount, account is charged the difference."""

    monthly_platform_fee: Annotated[
        MonthlyPlatformFee, pydantic.Field(alias="monthlyPlatformFee")
    ]
    r"""Fixed recurring amount paid in the billing period regardless of usage."""

    created_at: Annotated[datetime, pydantic.Field(alias="createdAt")]

    description: Optional[str] = None
    r"""A description of the fee plan."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    FeePlan.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .ticketcontact import TicketContact, TicketContactTypedDict
from .ticketstatus import TicketStatus
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TicketTypedDict(TypedDict):
    ticket_id: str
    number: int
    title: str
    contact: TicketContactTypedDict
    status: TicketStatus
    created_on: datetime
    updated_on: datetime
    latest_message_on: NotRequired[datetime]
    closed_on: NotRequired[datetime]
    foreign_id: NotRequired[str]


class Ticket(BaseModel):
    ticket_id: Annotated[str, pydantic.Field(alias="ticketID")]

    number: int

    title: str

    contact: TicketContact

    status: TicketStatus

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]

    latest_message_on: Annotated[
        Optional[datetime], pydantic.Field(alias="latestMessageOn")
    ] = None

    closed_on: Annotated[Optional[datetime], pydantic.Field(alias="closedOn")] = None

    foreign_id: Annotated[Optional[str], pydantic.Field(alias="foreignID")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["latestMessageOn", "closedOn", "foreignID"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Ticket.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class ApplePayMerchantDomainsTypedDict(TypedDict):
    account_id: str
    display_name: str
    r"""A UTF-8 string to display in the Buy button."""
    domains: List[str]
    r"""A unique list of fully-qualified, top-level or sub-domain names where you will accept Apple Pay."""
    created_on: datetime
    updated_on: datetime


class ApplePayMerchantDomains(BaseModel):
    account_id: Annotated[str, pydantic.Field(alias="accountID")]

    display_name: Annotated[str, pydantic.Field(alias="displayName")]
    r"""A UTF-8 string to display in the Buy button."""

    domains: List[str]
    r"""A unique list of fully-qualified, top-level or sub-domain names where you will accept Apple Pay."""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]


try:
    ApplePayMerchantDomains.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class PartnerFeesTypedDict(TypedDict):
    r"""Monthly partner costs that are charged separately and not included in residual subtotal (e.g. platform fees, minimums)."""

    minimum_commitment: AmountDecimalTypedDict
    r"""The minimum spending amount that must be met in the billing period. If actual usage is below the minimum amount, account is charged the difference."""
    monthly_platform: AmountDecimalTypedDict
    r"""Fixed recurring fee for the billing period regardless of usage."""
    total: AmountDecimalTypedDict
    r"""Total partner fees."""


class PartnerFees(BaseModel):
    r"""Monthly partner costs that are charged separately and not included in residual subtotal (e.g. platform fees, minimums)."""

    minimum_commitment: Annotated[
        AmountDecimal, pydantic.Field(alias="minimumCommitment")
    ]
    r"""The minimum spending amount that must be met in the billing period. If actual usage is below the minimum amount, account is charged the difference."""

    monthly_platform: Annotated[AmountDecimal, pydantic.Field(alias="monthlyPlatform")]
    r"""Fixed recurring fee for the billing period regardless of usage."""

    total: AmountDecimal
    r"""Total partner fees."""


try:
    PartnerFees.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .documenttype import DocumentType
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DocumentTypedDict(TypedDict):
    document_id: str
    type: DocumentType
    r"""Types of documents that can be uploaded."""
    content_type: str
    r"""The document's MIME type."""
    uploaded_at: datetime
    parse_errors: NotRequired[List[str]]
    r"""Optional array of errors encountered during automated parsing."""


class Document(BaseModel):
    document_id: Annotated[str, pydantic.Field(alias="documentID")]

    type: DocumentType
    r"""Types of documents that can be uploaded."""

    content_type: Annotated[str, pydantic.Field(alias="contentType")]
    r"""The document's MIME type."""

    uploaded_at: Annotated[datetime, pydantic.Field(alias="uploadedAt")]

    parse_errors: Annotated[
        Optional[List[str]], pydantic.Field(alias="parseErrors")
    ] = None
    r"""Optional array of errors encountered during automated parsing."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["parseErrors"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Document.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookdata import WebhookData, WebhookDataTypedDict
from .webhookeventtype import WebhookEventType
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WebhookEventTypedDict(TypedDict):
    r"""Webhook events are sent to your webhook URL when certain actions occur in the Moov API. You can subscribe to these events to receive real-time notifications."""

    event_id: str
    r"""Unique identifier for the webhook event."""
    type: WebhookEventType
    r"""The type of event that occurred."""
    data: WebhookDataTypedDict
    r"""The data for the webhook event. The contents are based on the event type."""
    created_on: datetime


class WebhookEvent(BaseModel):
    r"""Webhook events are sent to your webhook URL when certain actions occur in the Moov API. You can subscribe to these events to receive real-time notifications."""

    event_id: Annotated[str, pydantic.Field(alias="eventID")]
    r"""Unique identifier for the webhook event."""

    type: WebhookEventType
    r"""The type of event that occurred."""

    data: WebhookData
    r"""The data for the webhook event. The contents are based on the event type."""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]


try:
    WebhookEvent.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.components import (
    addresserror as components_addresserror,
    birthdatevalidationerror as components_birthdatevalidationerror,
    governmentiderror as components_governmentiderror,
    individualnameerror as components_individualnameerror,
    phonenumbererror as components_phonenumbererror,
    representativeresponsibilitieserror as components_representativeresponsibilitieserror,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ErrorTypedDict(TypedDict):
    name: NotRequired[components_individualnameerror.IndividualNameErrorTypedDict]
    phone: NotRequired[components_phonenumbererror.PhoneNumberErrorTypedDict]
    email: NotRequired[str]
    address: NotRequired[components_addresserror.AddressErrorTypedDict]
    birth_date: NotRequired[
        components_birthdatevalidationerror.BirthDateValidationErrorTypedDict
    ]
    government_id: NotRequired[components_governmentiderror.GovernmentIDErrorTypedDict]
    responsibilities: NotRequired[
        components_representativeresponsibilitieserror.RepresentativeResponsibilitiesErrorTypedDict
    ]


class Error(BaseModel):
    name: Optional[components_individualnameerror.IndividualNameError] = None

    phone: Optional[components_phonenumbererror.PhoneNumberError] = None

    email: Optional[str] = None

    address: Optional[components_addresserror.AddressError] = None

    birth_date: Annotated[
        Optional[components_birthdatevalidationerror.BirthDateValidationError],
        pydantic.Field(alias="birthDate"),
    ] = None

    government_id: Annotated[
        Optional[components_governmentiderror.GovernmentIDError],
        pydantic.Field(alias="governmentID"),
    ] = None

    responsibilities: Optional[
        components_representativeresponsibilitieserror.RepresentativeResponsibilitiesError
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "name",
                "phone",
                "email",
                "address",
                "birthDate",
                "governmentID",
                "responsibilities",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class RepresentativeValidationErrorData(BaseModel):
    error: Error


@dataclass(unsafe_hash=True)
class RepresentativeValidationError(MoovError):
    data: RepresentativeValidationErrorData = field(hash=False)

    def __init__(
        self,
        data: RepresentativeValidationErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


try:
    Error.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class ACHPaymentSettingsTypedDict(TypedDict):
    company_name: str
    r"""The description that shows up on ACH transactions. This will default to the account's display name on account creation."""


class ACHPaymentSettings(BaseModel):
    company_name: Annotated[str, pydantic.Field(alias="companyName")]
    r"""The description that shows up on ACH transactions. This will default to the account's display name on account creation."""


try:
    ACHPaymentSettings.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class FileStatus(str, Enum):
    r"""The file's status."""

    PENDING = "pending"
    APPROVED = "approved"
    REJECTED = "rejected"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .transferpaymentmethodscard import (
    TransferPaymentMethodsCard,
    TransferPaymentMethodsCardTypedDict,
)
from enum import Enum
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class PushToCardTransferPaymentMethodPaymentMethodType(str, Enum):
    PUSH_TO_CARD = "push-to-card"


class PushToCardTransferPaymentMethodTypedDict(TypedDict):
    payment_method_id: str
    r"""ID of the payment method."""
    payment_method_type: PushToCardTransferPaymentMethodPaymentMethodType
    card: TransferPaymentMethodsCardTypedDict
    r"""A card as contained within a payment method."""


class PushToCardTransferPaymentMethod(BaseModel):
    payment_method_id: Annotated[str, pydantic.Field(alias="paymentMethodID")]
    r"""ID of the payment method."""

    payment_method_type: Annotated[
        PushToCardTransferPaymentMethodPaymentMethodType,
        pydantic.Field(alias="paymentMethodType"),
    ]

    card: TransferPaymentMethodsCard
    r"""A card as contained within a payment method."""


try:
    PushToCardTransferPaymentMethod.model_rebuild()
except NameError:
    pass
